/* Datei:  test_comsf.c
 * Author: gkemnitz
 * Datum:  27.10.2015
 *
 * Beispiel zu Foliensatz IW4
 * Testprogramm fr die Teiber
 * comsf_pc, comsf_sonar und comsf_lcd
 *
 * PModLS an JD oben, 
 * Sonar an JD unten; Jumper JDX gekreuzt
 * PmodUSBUSART an JH oben, Jumper JHX gekreuzt
 */

#include "comsf_pc.h"
#include "comsf_sonar.h"
#include "comsf_lcd.h"

// LCD-Initialisierungstext
#define LCD_STR "S:... I:.... L:.s:... i:.. E:..."
// Anzeigepositionen fr Zahlen und Texte
#define LCP_SON	     2               // Sonarwert "S:..."
#define LCP_RMSG     8               // Eingabedaten "I:...."
#define LCP_LBYTE   15               // zuletzt empf. Byte
#define LCP_TIME    18               // Anz. Son.-Mess./50
#define LCP_ICT     24               // Anzahl Eingaben

//Anzeigepositionen fr Fehlerzaehler
#define LCP_COMERR  29               // Kommunikationsfehler
#define LCP_TESTERR 30               // Testfehlerzhler

int main(void){
  DDRG = 0xFF; PORTG = 0;
  uint8_t mrmsg[COM_PC_RMSG_LEN];    // Empfangsnachricht
  uint8_t msmsg[COM_PC_SMSG_LEN];    // Sendnachricht
  uint16_t snrval, sct=0, ict=0;
	
  sonar_init();	                     // alle Treiber 
  com_pc_init();                     // initialisieren
  lcd_init((uint8_t*)LCD_STR);
  while(1){
    PORTG = 1;
    lcd_step();	                     // Schrittfunktionen 
    PORTG = 2;                       // aller Treiber
    com_pc_step();                   // aufrufen
    PORTG = 3;
    sonar_step();
    PORTG = 0;
    if (sonar_get(&snrval)){         // wenn neue Sonardaten
      PORTG = 4;
      // Sonarwert und Messwertnummer auf LCD schreiben
      lcd_disp_val(snrval, LCP_SON, 3);  // Sonarwert
      sct++;   //Zaehler Sonarmessungen erhoehen und Wert/50
      lcd_disp_val(sct/50, LCP_TIME, 3); // ausgeben
    }
    if (com_pc_get(mrmsg)){         // Wenn neue PC-Nachricht
      PORTG = 5;
      // diese in mrmsg bernehmen, auf LCD ausgeben
      lcd_disp_str(mrmsg, LCP_RMSG, COM_PC_SMSG_LEN);
      lcd_incErr(LCP_TESTERR);	    // Testfehlerzhler erhhen
      // Eingabenzhler erhhen und ausgeben
      lcd_disp_val(++ict, LCP_ICT, 2);	 
      msmsg[0] = snrval >> 8;       // Sensorwert und Messwert-
      msmsg[1] = snrval & 0xFF;     // nummer byteweise in den
      msmsg[2] = sct >> 8;          // string "msg" schreiben
      msmsg[3] = sct & 0xFF;
      if (!com_pc_send(msmsg))      // "msmsg" versenden
        lcd_incErr(LCP_COMERR);     // wenn senden erfolglos
    }                               // Sendfehlerzhler erhhen
		// immer zuletzt empf. Byte auf LCD schreiben
    lcd_disp_chr(com_pc_last_byte(), LCP_LBYTE);                          
  }  
}
